"""
    Slixmpp: The Slick XMPP Library
    Implementation of HTTP over XMPP transport
    http://xmpp.org/extensions/xep-0332.html
    Copyright (C) 2015 Riptide IO, sangeeth@riptideio.com
    This file is part of slixmpp.

    See the file LICENSE for copying permission.
"""

from slixmpp.plugins.xep_0332.stanza.request import HTTPRequest
from slixmpp.plugins.xep_0332.stanza.response import HTTPResponse
from slixmpp.plugins.xep_0332.stanza.data import HTTPData
