"""
    Slixmpp: The Slick XMPP Library
    Copyright (C) 2011  Nathanael C. Fritz
    This file is part of Slixmpp.

    See the file LICENSE for copying permission.
"""


from slixmpp.features.feature_mechanisms.stanza.mechanisms import Mechanisms
from slixmpp.features.feature_mechanisms.stanza.auth import Auth
from slixmpp.features.feature_mechanisms.stanza.success import Success
from slixmpp.features.feature_mechanisms.stanza.failure import Failure
from slixmpp.features.feature_mechanisms.stanza.challenge import Challenge
from slixmpp.features.feature_mechanisms.stanza.response import Response
from slixmpp.features.feature_mechanisms.stanza.abort import Abort
